@@ -32,7 +32,6 @@
 
 #define __GCK_INSIDE_HEADER__
 
-#include "gck-deprecated.h"
 #include "gck-enum-types.h"
 
 G_BEGIN_DECLS
@@ -144,7 +143,7 @@ void                gck_attribute_init_copy                 (GckAttribute *dest,
 GType               gck_attribute_get_type                  (void) G_GNUC_CONST;
 
 GckAttribute*       gck_attribute_new                       (gulong attr_type,
-                                                             gpointer value,
+                                                             const guchar *value,
                                                              gsize length);
 
 GckAttribute*       gck_attribute_new_invalid               (gulong attr_type);
@@ -163,15 +162,15 @@ GckAttribute*       gck_attribute_new_ulong                 (gulong attr_type,
 GckAttribute*       gck_attribute_new_string                (gulong attr_type,
                                                              const gchar *value);
 
-gboolean            gck_attribute_is_invalid                (GckAttribute *attr);
+gboolean            gck_attribute_is_invalid                (const GckAttribute *attr);
 
-gboolean            gck_attribute_get_boolean               (GckAttribute *attr);
+gboolean            gck_attribute_get_boolean               (const GckAttribute *attr);
 
-gulong              gck_attribute_get_ulong                 (GckAttribute *attr);
+gulong              gck_attribute_get_ulong                 (const GckAttribute *attr);
 
-gchar*              gck_attribute_get_string                (GckAttribute *attr);
+gchar*              gck_attribute_get_string                (const GckAttribute *attr);
 
-void                gck_attribute_get_date                  (GckAttribute *attr,
+void                gck_attribute_get_date                  (const GckAttribute *attr,
                                                              GDate* value);
 
 gboolean            gck_attribute_equal                     (gconstpointer attr1,
@@ -179,108 +178,208 @@ gboolean            gck_attribute_equal                     (gconstpointer attr1
 
 guint               gck_attribute_hash                      (gconstpointer attr);
 
-GckAttribute*       gck_attribute_dup                       (GckAttribute *attr);
+GckAttribute*       gck_attribute_dup                       (const GckAttribute *attr);
 
 void                gck_attribute_clear                     (GckAttribute *attr);
 
 void                gck_attribute_free                      (gpointer attr);
 
-void                gck_attribute_dump                      (GckAttribute *attr);
+void                gck_attribute_dump                      (const GckAttribute *attr);
+
+typedef struct _GckBuilder GckBuilder;
+
+struct _GckBuilder {
+	/*< private >*/
+	gsize x[16];
+};
+
+typedef enum {
+	GCK_BUILDER_NONE,
+	GCK_BUILDER_SECURE_MEMORY = 1,
+} GckBuilderFlags;
 
 typedef struct _GckAttributes GckAttributes;
 
-#define             GCK_TYPE_ATTRIBUTES                     (gck_attributes_get_boxed_type ())
+GckBuilder *         gck_builder_new                        (GckBuilderFlags flags);
 
-GType               gck_attributes_get_type                 (void) G_GNUC_CONST;
+#define              GCK_BUILDER_INIT                       { { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } }
 
-GckAttributes*      gck_attributes_new                      (void);
+GckBuilder *         gck_builder_ref                        (GckBuilder *builder);
 
-GckAttributes*      gck_attributes_new_empty                (gulong attr_type,
-                                                             ...);
+void                 gck_builder_unref                      (gpointer builder);
 
-GckAttributes*      gck_attributes_new_full                 (GckAllocator allocator);
+void                 gck_builder_init                       (GckBuilder *builder);
 
-GckAttribute*       gck_attributes_at                       (GckAttributes *attrs,
-                                                             guint index);
+void                 gck_builder_init_full                  (GckBuilder *builder,
+                                                             GckBuilderFlags flags);
 
-GckAttribute*       gck_attributes_add                      (GckAttributes *attrs,
-                                                             GckAttribute *attr);
+#define              GCK_TYPE_BUILDER                       (gck_builder_get_type ())
 
-void                gck_attributes_add_all                  (GckAttributes *attrs,
-                                                             GckAttributes *from);
+GType                gck_builder_get_type                   (void) G_GNUC_CONST;
 
-GckAttribute*       gck_attributes_add_data                 (GckAttributes *attrs,
+void                 gck_builder_take_data                  (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             guchar *value,
+                                                             gsize length);
+
+void                 gck_builder_add_data                   (GckBuilder *builder,
                                                              gulong attr_type,
                                                              const guchar *value,
                                                              gsize length);
 
-GckAttribute*       gck_attributes_add_invalid              (GckAttributes *attrs,
+void                 gck_builder_add_empty                  (GckBuilder *builder,
                                                              gulong attr_type);
 
-GckAttribute*       gck_attributes_add_empty                (GckAttributes *attrs,
+void                 gck_builder_add_invalid                (GckBuilder *builder,
                                                              gulong attr_type);
 
-GckAttribute*       gck_attributes_add_boolean              (GckAttributes *attrs,
-                                                             gulong attr_type,
-                                                             gboolean value);
-
-GckAttribute*       gck_attributes_add_string               (GckAttributes *attrs,
-                                                             gulong attr_type,
-                                                             const gchar *value);
-
-GckAttribute*       gck_attributes_add_date                 (GckAttributes *attrs,
-                                                             gulong attr_type,
-                                                             const GDate *value);
-
-GckAttribute*       gck_attributes_add_ulong                (GckAttributes *attrs,
+void                 gck_builder_add_ulong                  (GckBuilder *builder,
                                                              gulong attr_type,
                                                              gulong value);
 
-GckAttribute*       gck_attributes_find                     (GckAttributes *attrs,
+void                 gck_builder_add_boolean                (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             gboolean value);
+
+void                 gck_builder_add_date                   (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             const GDate *value);
+
+void                 gck_builder_add_string                 (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             const gchar *value);
+
+void                 gck_builder_add_attribute              (GckBuilder *builder,
+                                                             const GckAttribute *attr);
+
+void                 gck_builder_add_owned                  (GckBuilder *builder,
+                                                             const GckAttribute *attr);
+
+void                 gck_builder_add_all                    (GckBuilder *builder,
+                                                             GckAttributes *attrs);
+
+void                 gck_builder_add_only                   (GckBuilder *builder,
+                                                             GckAttributes *attrs,
+                                                             gulong only_type,
+                                                             ...);
+
+void                 gck_builder_add_onlyv                  (GckBuilder *builder,
+                                                             GckAttributes *attrs,
+                                                             const gulong *only_types,
+                                                             guint n_only_types);
+
+void                 gck_builder_add_except                 (GckBuilder *builder,
+                                                             GckAttributes *attrs,
+                                                             gulong except_type,
+                                                             ...);
+
+void                 gck_builder_add_exceptv                (GckBuilder *builder,
+                                                             GckAttributes *attrs,
+                                                             const gulong *except_types,
+                                                             guint n_except_types);
+
+void                 gck_builder_set_data                   (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             const guchar *value,
+                                                             gsize length);
+
+void                 gck_builder_set_empty                  (GckBuilder *builder,
                                                              gulong attr_type);
 
-gboolean            gck_attributes_find_boolean             (GckAttributes *attrs,
+void                 gck_builder_set_invalid                (GckBuilder *builder,
+                                                             gulong attr_type);
+
+void                 gck_builder_set_ulong                  (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             gulong value);
+
+void                 gck_builder_set_boolean                (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             gboolean value);
+
+void                 gck_builder_set_date                   (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             const GDate *value);
+
+void                 gck_builder_set_string                 (GckBuilder *builder,
+                                                             gulong attr_type,
+                                                             const gchar *value);
+
+void                 gck_builder_set_all                    (GckBuilder *builder,
+                                                             GckAttributes *attrs);
+
+const GckAttribute * gck_builder_find                       (GckBuilder *builder,
+                                                             gulong attr_type);
+
+gboolean             gck_builder_find_boolean               (GckBuilder *builder,
                                                              gulong attr_type,
                                                              gboolean *value);
 
-gboolean            gck_attributes_find_ulong               (GckAttributes *attrs,
+gboolean             gck_builder_find_ulong                 (GckBuilder *builder,
                                                              gulong attr_type,
                                                              gulong *value);
 
-gboolean            gck_attributes_find_string              (GckAttributes *attrs,
+gboolean             gck_builder_find_string                (GckBuilder *builder,
                                                              gulong attr_type,
                                                              gchar **value);
 
-gboolean            gck_attributes_find_date                (GckAttributes *attrs,
+gboolean             gck_builder_find_date                  (GckBuilder *builder,
                                                              gulong attr_type,
                                                              GDate *value);
 
-void                gck_attributes_set_boolean              (GckAttributes *attrs,
+GckAttributes *      gck_builder_steal                      (GckBuilder *builder);
+
+GckAttributes *      gck_builder_end                        (GckBuilder *builder);
+
+GckBuilder *         gck_builder_copy                       (GckBuilder *builder);
+
+void                 gck_builder_clear                      (GckBuilder *builder);
+
+#define              GCK_TYPE_ATTRIBUTES                    (gck_attributes_get_boxed_type ())
+
+GType                gck_attributes_get_type                (void) G_GNUC_CONST;
+
+GckAttributes *      gck_attributes_new                     (gulong reserved);
+
+GckAttributes *      gck_attributes_new_empty               (gulong first_type,
+                                                             ...);
+
+const GckAttribute * gck_attributes_at                      (GckAttributes *attrs,
+                                                             guint index);
+
+const GckAttribute * gck_attributes_find                    (GckAttributes *attrs,
+                                                             gulong attr_type);
+
+gboolean             gck_attributes_find_boolean            (GckAttributes *attrs,
                                                              gulong attr_type,
-                                                             gboolean value);
+                                                             gboolean *value);
 
-void                gck_attributes_set_ulong                (GckAttributes *attrs,
+gboolean             gck_attributes_find_ulong              (GckAttributes *attrs,
                                                              gulong attr_type,
-                                                             gulong value);
+                                                             gulong *value);
 
-void                gck_attributes_set_string               (GckAttributes *attrs,
+gboolean             gck_attributes_find_string             (GckAttributes *attrs,
                                                              gulong attr_type,
-                                                             const gchar *value);
+                                                             gchar **value);
 
-void                gck_attributes_set_date                 (GckAttributes *attrs,
+gboolean             gck_attributes_find_date               (GckAttributes *attrs,
                                                              gulong attr_type,
-                                                             const GDate *value);
+                                                             GDate *value);
 
-gulong              gck_attributes_count                    (GckAttributes *attrs);
+gulong               gck_attributes_count                   (GckAttributes *attrs);
 
-GckAttributes*      gck_attributes_ref                      (GckAttributes *attrs);
+GckAttributes *      gck_attributes_ref                     (GckAttributes *attrs);
 
-void                gck_attributes_unref                    (gpointer attrs);
+GckAttributes *      gck_attributes_ref_sink                (GckAttributes *attrs);
 
-gboolean            gck_attributes_contains                 (GckAttributes *attrs,
-                                                             GckAttribute *match);
+void                 gck_attributes_unref                   (gpointer attrs);
 
-void                gck_attributes_dump                     (GckAttributes *attrs);
+gboolean             gck_attributes_contains                (GckAttributes *attrs,
+                                                             const GckAttribute *match);
+
+void                 gck_attributes_dump                    (GckAttributes *attrs);
+
+gchar *              gck_attributes_to_string               (GckAttributes *attrs);
 
 /* -------------------------------------------------------------------------
  * FORWARDS
@@ -289,7 +388,7 @@ typedef struct _GckSlot GckSlot;
 typedef struct _GckModule GckModule;
 typedef struct _GckSession GckSession;
 typedef struct _GckObject GckObject;
-typedef struct _GckObjectAttributes GckObjectAttributes;
+typedef struct _GckObjectCache GckObjectCache;
 typedef struct _GckEnumerator GckEnumerator;
 typedef struct _GckUriData GckUriData;
 
@@ -1218,13 +1317,13 @@ GckAttributes*      gck_object_get                          (GckObject *self,
                                                              ...);
 
 GckAttributes*      gck_object_get_full                     (GckObject *self,
-                                                             gulong *attr_types,
+                                                             const gulong *attr_types,
                                                              guint n_attr_types,
                                                              GCancellable *cancellable,
                                                              GError **error);
 
 void                gck_object_get_async                    (GckObject *self,
-                                                             gulong *attr_types,
+                                                             const gulong *attr_types,
                                                              guint n_attr_types,
                                                              GCancellable *cancellable,
                                                              GAsyncReadyCallback callback,
@@ -1295,29 +1394,69 @@ GckAttributes*      gck_object_get_template_finish          (GckObject *self,
  * OBJECT ATTRIBUTES
  */
 
-#define GCK_TYPE_OBJECT_ATTRIBUTES                    (gck_object_attributes_get_type ())
-#define GCK_OBJECT_ATTRIBUTES(obj)                    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GCK_TYPE_OBJECT_ATTRIBUTES, GckObjectAttributes))
-#define GCK_IS_OBJECT_ATTRIBUTES(obj)                 (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GCK_TYPE_OBJECT_ATTRIBUTES))
-#define GCK_OBJECT_ATTRIBUTES_GET_INTERFACE(inst)     (G_TYPE_INSTANCE_GET_INTERFACE ((inst), GCK_TYPE_OBJECT_ATTRIBUTES, GckObjectAttributesIface))
+#define GCK_TYPE_OBJECT_CACHE                    (gck_object_cache_get_type ())
+#define GCK_OBJECT_CACHE(obj)                    (G_TYPE_CHECK_INSTANCE_CAST ((obj), GCK_TYPE_OBJECT_CACHE, GckObjectCache))
+#define GCK_IS_OBJECT_CACHE(obj)                 (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GCK_TYPE_OBJECT_CACHE))
+#define GCK_OBJECT_CACHE_GET_INTERFACE(inst)     (G_TYPE_INSTANCE_GET_INTERFACE ((inst), GCK_TYPE_OBJECT_CACHE, GckObjectCacheIface))
 
-typedef struct _GckObjectAttributesIface GckObjectAttributesIface;
+typedef struct _GckObjectCacheIface GckObjectCacheIface;
 
-struct _GckObjectAttributesIface {
+struct _GckObjectCacheIface {
 	GTypeInterface interface;
 
-	const gulong *attribute_types;
-	gint n_attribute_types;
+	const gulong *  default_types;
+	gint            n_default_types;
+
+	void         (* fill)                              (GckObjectCache *object,
+	                                                    GckAttributes *attrs);
 
 	/*< private >*/
 	gpointer reserved[6];
 };
 
-GType               gck_object_attributes_get_type          (void) G_GNUC_CONST;
+GType               gck_object_cache_get_type              (void) G_GNUC_CONST;
 
-GckAttributes *     gck_object_attributes_get_attributes    (GckObjectAttributes *object);
+GckAttributes *     gck_object_cache_get_attributes        (GckObjectCache *object);
 
-void                gck_object_attributes_set_attributes    (GckObjectAttributes *object,
-                                                             GckAttributes *attributes);
+void                gck_object_cache_set_attributes        (GckObjectCache *object,
+                                                            GckAttributes *attrs);
+
+void                gck_object_cache_fill                  (GckObjectCache *object,
+                                                            GckAttributes *attrs);
+
+gboolean            gck_object_cache_update                (GckObjectCache *object,
+                                                            const gulong *attr_types,
+                                                            gint n_attr_types,
+                                                            GCancellable *cancellable,
+                                                            GError **error);
+
+void                gck_object_cache_update_async          (GckObjectCache *object,
+                                                            const gulong *attr_types,
+                                                            gint n_attr_types,
+                                                            GCancellable *cancellable,
+                                                            GAsyncReadyCallback callback,
+                                                            gpointer user_data);
+
+gboolean            gck_object_cache_update_finish         (GckObjectCache *object,
+                                                            GAsyncResult *result,
+                                                            GError **error);
+
+GckAttributes *     gck_object_cache_lookup                (GckObject *object,
+                                                            const gulong *attr_types,
+                                                            gint n_attr_types,
+                                                            GCancellable *cancellable,
+                                                            GError **error);
+
+void                gck_object_cache_lookup_async          (GckObject *object,
+                                                            const gulong *attr_types,
+                                                            gint n_attr_types,
+                                                            GCancellable *cancellable,
+                                                            GAsyncReadyCallback callback,
+                                                            gpointer user_data);
+
+GckAttributes *     gck_object_cache_lookup_finish         (GckObject *object,
+                                                            GAsyncResult *result,
+                                                            GError **error);
 
 /* ------------------------------------------------------------------------
  * PASSWORD
@@ -1417,6 +1556,8 @@ void                gck_uri_data_free                       (GckUriData *uri_dat
 
 G_END_DECLS
 
+#include "gck-deprecated.h"
+
 #undef __GCK_INSIDE_HEADER__
 
 #endif /* GCK_H */
