@@ -23,6 +23,8 @@
 
 #include "config.h"
 
+#define DEBUG_FLAG GCR_DEBUG_IMPORT
+#include "gcr-debug.h"
 #include "gcr-deprecated-base.h"
 #include "gcr-importer.h"
 #include "gcr-internal.h"
@@ -119,6 +121,15 @@ gcr_importer_default_init (GcrImporterIface *iface)
 		                              "Interaction for prompts",
 		                              G_TYPE_TLS_INTERACTION, G_PARAM_READWRITE));
 
+		/**
+		 * GcrImporter:uri:
+		 *
+		 * The URI of the location imported to.
+		 */
+		g_object_interface_install_property (iface,
+		                g_param_spec_string ("uri", "URI", "URI of location",
+		                                     NULL, G_PARAM_READABLE));
+
 		g_once_init_leave (&initialized, 1);
 	}
 }
@@ -216,10 +227,16 @@ gcr_importer_create_for_parsed (GcrParsed *parsed)
 	if (attrs != NULL)
 		gck_attributes_ref (attrs);
 	else
-		attrs = gck_attributes_new ();
+		attrs = gck_attributes_new_empty (GCK_INVALID);
 
 	seen = g_hash_table_new (g_direct_hash, g_direct_equal);
 
+	if (_gcr_debugging) {
+		gchar *a = gck_attributes_to_string (attrs);
+		_gcr_debug ("looking for importer for: %s", a);
+		g_free (a);
+	}
+
 	for (i = 0; i < registered_importers->len; ++i) {
 		registered = &(g_array_index (registered_importers, GcrRegistered, i));
 		n_attrs = gck_attributes_count (registered->attrs);
@@ -233,6 +250,12 @@ gcr_importer_create_for_parsed (GcrParsed *parsed)
 			}
 		}
 
+		if (_gcr_debugging) {
+			gchar *a = gck_attributes_to_string (registered->attrs);
+			_gcr_debug ("importer %s: %s", matched ? "matched" : "didn't match", a);
+			g_free (a);
+		}
+
 		if (matched) {
 			if (check_if_seen_or_add (seen, GUINT_TO_POINTER (registered->importer_type)))
 				continue;
@@ -376,8 +399,15 @@ gcr_importer_import (GcrImporter *importer,
 	g_return_val_if_fail (iface->import_finish != NULL, FALSE);
 
 	closure = g_new0 (ImportClosure, 1);
+#if GLIB_CHECK_VERSION(2,31,2)
+	closure->cond = g_new (GCond, 1);
+	g_cond_init (closure->cond);
+	closure->mutex = g_new (GMutex, 1);
+	g_mutex_init (closure->mutex);
+#else
 	closure->cond = g_cond_new ();
 	closure->mutex = g_mutex_new ();
+#endif
 	closure->context = g_main_context_get_thread_default ();
 	g_mutex_lock (closure->mutex);
 
@@ -412,8 +442,15 @@ gcr_importer_import (GcrImporter *importer,
 	if (closure->error)
 		g_propagate_error (error, closure->error);
 
+#if GLIB_CHECK_VERSION(2,31,2)
+	g_cond_clear (closure->cond);
+	g_free (closure->cond);
+	g_mutex_clear (closure->mutex);
+	g_free (closure->mutex);
+#else
 	g_cond_free (closure->cond);
 	g_mutex_free (closure->mutex);
+#endif
 	g_free (closure);
 
 	return result;
@@ -598,6 +635,8 @@ gcr_importer_set_slot (GcrImporter *self,
  * gcr_importer_get_prompt_behavior:
  * @self: The importer
  *
+ * Does nothing.
+ *
  * Returns: zero
  *
  * Deprecated: since 3.4.0
